
% WHAT IF...? OUTPUT LAYER

% FIG 4

% NUCLEAR SOMATIC CHARGE DRIVEN BY 40 PCs. CURRENT DRIVEN BY EACH SPIKE 
% HAS A TIME CONSTANT OF 1.15 ms (BLUE), 2.05 ms (YELLOW), % 3.95 ms (PINK), 
% 6.05 ms (GREEN) AND 7.95 ms (GREY). PC RATE: 50 Hz.

% Simulate one-quantum-per-spike charge entry to the nulcear soma

% Create the shape of charge entry with a peak-and-tail signature of trans-
% membrane currrent, with an adjustable time constant. It is unnecessary to 
% also adjust peak amplitude because the functional relevance of changing 
% either is through the effect it has on the shape of the signature, and 
% and in particular the ratio of amplitude to time constant. 

opengl software
figure;

x = -0.7:0.05:20;

gaussian1 = @(x) (1/sqrt((2*pi))*exp(-x.^2/2));% a normal distribution
skewedgaussian1 = @(x,alpha) 2*gaussian1(x).*normcdf(alpha*x);% that normal 
% distribution skewed

g2_amplitude = .93; % amplitude - changing the amplitude doesn't change the rise 
% time to peak amplitude. amp is only used to compensate for an effect on
% amplitude of dur.
g2_SDproxy = 6; % duration coefficient - increasing the value increases the time 
% constant of skewedgaussian (by a diminishing ratio i.e. a linear increase 
% of time constant requires an exponential increase of SDproxy).

gaussian2 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g2_SDproxy))*g2_amplitude);% the same 
% distribution as gaussian1 but with SD adjusted by SDproxy
skewedgaussian2 = @(x,alpha) 2*gaussian2(x).*normcdf(alpha*x);% skewed

g3_amplitude = .9;
g3_SDproxy = 20;

gaussian3 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g3_SDproxy))*g3_amplitude);
skewedgaussian3 = @(x,alpha) 2*gaussian3(x).*normcdf(alpha*x);

g4_amplitude = .88;
g4_SDproxy = 44;

gaussian4 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g4_SDproxy))*g4_amplitude);
skewedgaussian4 = @(x,alpha) 2*gaussian4(x).*normcdf(alpha*x);

g5_amplitude = .88;
g5_SDproxy = 74;

gaussian5 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g5_SDproxy))*g5_amplitude);
skewedgaussian5 = @(x,alpha) 2*gaussian5(x).*normcdf(alpha*x);

% % plot(x, skewedgaussian1(x, 4))
% % hold on
% % % plot(x, gaussian1(x))
% % % plot(x, gaussian2(x))
% % % plot(x, gaussian3(x))
% % % plot(x, gaussian4(x))
% % % plot(x, gaussian5(x))
% % plot(x, skewedgaussian2(x, 4))
% % plot(x, skewedgaussian3(x, 4))
% % plot(x, skewedgaussian4(x, 4))
% % plot(x, skewedgaussian5(x, 4))

% DERIVE TIME CONSTANTS
g1_influxUnit = skewedgaussian1(x, 4); % an array of y values in the range for x, 
% set above, representing charge influx driven by a single spike
[M,I] = max(g1_influxUnit); % M is the peak value in array and I its index position
drop = M*.368; % a 63.2% reduction of M
[~,closestIndex] = min(abs(drop-g1_influxUnit(I:end))); % Finds the index position 
% of the nearest value to 'drop' in the array 'I:end'
g1_timeConstant = closestIndex/20 % There are 20 index positions per ms, so 
% dividing the distance (measured in index positions) between I and
% closestIndex by 20 gives the time interval between the peak value of
% the skewed distribution and a reduction by 63.2%

g2_influxUnit = skewedgaussian2(x, 4);
[M,I] = max(g2_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g2_influxUnit(I:end)));
g2_timeConstant = closestIndex/20

g3_influxUnit = skewedgaussian3(x, 4);
[M,I] = max(g3_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g3_influxUnit(I:end)));
g3_timeConstant = closestIndex/20

g4_influxUnit = skewedgaussian4(x, 4);
[M,I] = max(g4_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g4_influxUnit(I:end)));
g4_timeConstant = closestIndex/20

g5_influxUnit = skewedgaussian5(x, 4);
[M,I] = max(g5_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g5_influxUnit(I:end)));
g5_timeConstant = closestIndex/20

% SUMMARY
% Create an array of 4,000 zeros representing a time interval of 200 ms
% divided into twentieths of a ms. An array represents a single PC. More 
% exactly, it will be used to represent charge influx driven by spike input 
% from a single PC. At each index position, there is a probability, P, of
% an input spike. At a steady afferent rate, it is the same in all moments. 
% When there is a spike, 0 is replaced by 0.001 at that index position.  
% Assume a 1 ms refractory period. Call the array PC1. Call the first such 
% index position index1. Call the next index2, and so on. Next, add 
% 'influx' to PC1 at index positions index1:index1+(length(influx)-1).
% Then, repeat at index positions index2:index2+(length(influx)-1), etc.

N = 50; % The number of simulated nuclear cells
Q = 400; % The number of simulated PCs 
R = 20; % Refractory period - 20 = 1 ms
S = 40; % The number of PCs afferent to a single nuclear cell
%%S = 300; % Maximum Hz

%%% PANEL 1 %%%

% % Hz = 50; % PC firing rate
% % p = Hz/20000; % The probability of a spike in any moment (a moment being 
% % % 1/20th of a ms.

% DATA FOR 1.15 ms TIME CONSTANT 

% % k = 1;
% % master = zeros([1 12000]); % 4,000 1/20ths of a ms - i.e., 200 ms, plus an 
% % % extra 20 ms so that if charge influx is triggered towards the end of the
% % % modelled period, the current signature is not truncated

% sine wave
t = 0.00025:0.00025:3; % x axis units representing time on an arbitrary 
% scale
a = 75; % adjusts rate curve amplitude i.e. vertical range of rate data on 
% either side of v
v = 125; % vertical adjustment of the rate curve - without this, the range
% of wave would be centred on zero
w = 2; % wavelength adjustment - a higher value gives shorter wavelength. 
% Set at 2, a whole wave fits in the range t = 0 to 1.
d = 40; % adjusts dynamics curve amplitude. A higher value gives a smaller 
% range. 

wave = a*sin(w*pi*t)+v;

% The following while loop will generate a 400 x 12000 matrix. Each row
% represents a PC. A row has 12000 elements each corresponding to 1/20th of
% a ms (so a total of 600 ms). The value at each index position 
% represents inhibitory current across the nuclear somatic membrane

%%DATA FOR 1.15 ms TIME CONSTANT 

k = 1;
PCpopulation = zeros([400 12000]);

while k < Q + 1

    PCspikes = zeros([1 12000]);

    for i = 1:12000
        Hz = wave(i);
        p = Hz/20000;
        r = rand;
        if r <= p
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 12000]);
    
    i = 1;
    while i < 12000
        if i < length(PCspikes)-(length(g1_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g1_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g1_influxUnit)-1)+g1_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCpopulation(k,:) = PCcurrent;

    k = k+1;

end

h = 1;

allremasters = zeros([1 12000]);

while h < N + 1

    j = 1;
    master = zeros([1 12000]);
    
    while j < S + 1
    
        master = master + PCpopulation(randi(Q),:); %
        j = j + 1;
    
    end
    
    remastered = integrate_all_intervals(master);

    if h < 11   
        subplot(3,1,[1 2])
        x = 1:12000;
        plot(x,remastered,'Color',([0 0 .6]))%,'LineWidth',1.4)
        xlabel('ms'); ylabel('current'); title('40:1 convergence, 50-200 Hz, 1 ms rolling mean');
        xlim([3600 11600]); xticks([3600 4100 4600 ...
            5100 5600 6100 6600 7100 7600 8100 8600 9100 9600 10100 10600 11100 ...
            11600]); 
        xticklabels([0 25 50 75 100 125 150 175 200 225 250 275 300 325 350 375 ...
            400])
        ylim([0 42]); yticks([0 10 20 30 40]); yticklabels([0 10 20 30 40])
        set(gca,'FontSize',10)
        
        hold on
    end
    
    h = h + 1;

    allremasters = [allremasters; remastered];

end

allremastersmean = mean(allremasters);
plot(x,allremastersmean,'Color',([0 0 .6]),'LineWidth',1.5)

%%DATA FOR 2.05 ms TIME CONSTANT 

k = 1;
PCpopulation = zeros([400 12000]);

while k < Q + 1

    PCspikes = zeros([1 12000]);

    for i = 1:12000
        Hz = wave(i);
        p = Hz/20000;
        r = rand;
        if r <= p
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 12000]);
    
    i = 1;
    while i < 12000
        if i < length(PCspikes)-(length(g2_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g2_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g2_influxUnit)-1)+g2_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCpopulation(k,:) = PCcurrent;

    k = k+1;

end

h = 1;

allremasters = zeros([1 12000]);

while h < N + 1

    j = 1;
    master = zeros([1 12000]);
    
    while j < S + 1
    
        master = master + PCpopulation(randi(Q),:); %
        j = j + 1;
    
    end
    
    remastered = integrate_all_intervals(master);
    
%     subplot(1,1,1)
%     x = 1:12000;
    if h < 11
        plot(x,remastered,'Color',([1 .84 0]))%,'LineWidth',1.4)
    end
%     xlabel('ms'); ylabel('current'); title('40:1 convergence, 50-200 Hz, 1 ms rolling av');
%     xlim([3600 11600]); xticks([3600 4100 4600 ...
%         5100 5600 6100 6600 7100 7600 8100 8600 9100 9600 10100 10600 11100 ...
%         11600]); 
%     xticklabels([0 25 50 75 100 125 150 175 200 225 250 275 300 325 350 375 ...
%         400])
%     ylim([0 30]); yticks([0 10 20 30]); yticklabels([0 10 20 30])
%     set(gca,'FontSize',10)
    
    hold on
    
    h = h + 1;

    allremasters = [allremasters; remastered];

end

allremastersmean = mean(allremasters);
plot(x,allremastersmean,'Color',([1 .84 0]),'LineWidth',1.5)

%%DATA FOR 3.95 ms TIME CONSTANT 

k = 1;
PCpopulation = zeros([400 12000]);

while k < Q + 1

    PCspikes = zeros([1 12000]);

    for i = 1:12000
        Hz = wave(i);
        p = Hz/20000;
        r = rand;
        if r <= p
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 12000]);
    
    i = 1;
    while i < 12000
        if i < length(PCspikes)-(length(g3_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g3_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g3_influxUnit)-1)+g3_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCpopulation(k,:) = PCcurrent;

    k = k+1;

end

h = 1;

allremasters = zeros([1 12000]);

while h < N + 1

    j = 1;
    master = zeros([1 12000]);
    
    while j < S + 1
    
        master = master + PCpopulation(randi(Q),:); %
        j = j + 1;
    
    end
    
    remastered = integrate_all_intervals(master);
    
%     subplot(1,1,1)
%     x = 1:12000;
    if h < 11
        plot(x,remastered,'Color',([1 .5 .5]))%,'LineWidth',1.4)
    end
%     xlabel('ms'); ylabel('current'); title('40:1 convergence, 50-200 Hz, 1 ms rolling av');
%     xlim([3600 11600]); xticks([3600 4100 4600 ...
%         5100 5600 6100 6600 7100 7600 8100 8600 9100 9600 10100 10600 11100 ...
%         11600]); 
%     xticklabels([0 25 50 75 100 125 150 175 200 225 250 275 300 325 350 375 ...
%         400])
%     ylim([0 30]); yticks([0 10 20 30]); yticklabels([0 10 20 30])
%     set(gca,'FontSize',10)
    
    hold on
    
    h = h + 1;

    allremasters = [allremasters; remastered];

end

allremastersmean = mean(allremasters);
plot(x,allremastersmean,'Color',([1 .5 .5]),'LineWidth',1.5)

%%DATA FOR 6.05 ms TIME CONSTANT 

k = 1;
PCpopulation = zeros([400 12000]);

while k < Q + 1

    PCspikes = zeros([1 12000]);

    for i = 1:12000
        Hz = wave(i);
        p = Hz/20000;
        r = rand;
        if r <= p
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 12000]);
    
    i = 1;
    while i < 12000
        if i < length(PCspikes)-(length(g4_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g4_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g4_influxUnit)-1)+g4_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCpopulation(k,:) = PCcurrent;

    k = k+1;

end

h = 1;

allremasters = zeros([1 12000]);

while h < N + 1

    j = 1;
    master = zeros([1 12000]);
    
    while j < S + 1
    
        master = master + PCpopulation(randi(Q),:); %
        j = j + 1;
    
    end
    
    remastered = integrate_all_intervals(master);
    
%     subplot(1,1,1)
%     x = 1:12000;
    if h < 11
        plot(x,remastered,'Color',([.42 .56 .14]))%,'LineWidth',1.4)
    end
%     xlabel('ms'); ylabel('current'); title('40:1 convergence, 50-200 Hz, 1 ms rolling av');
%     xlim([3600 11600]); xticks([3600 4100 4600 ...
%         5100 5600 6100 6600 7100 7600 8100 8600 9100 9600 10100 10600 11100 ...
%         11600]); 
%     xticklabels([0 25 50 75 100 125 150 175 200 225 250 275 300 325 350 375 ...
%         400])
%     ylim([0 30]); yticks([0 10 20 30]); yticklabels([0 10 20 30])
%     set(gca,'FontSize',10)
    
    hold on
    
    h = h + 1;

    allremasters = [allremasters; remastered];

end

allremastersmean = mean(allremasters);
plot(x,allremastersmean,'Color',([.42 .56 .14]),'LineWidth',1.5)

%%DATA FOR 7.95 ms TIME CONSTANT 

k = 1;
PCpopulation = zeros([400 12000]);

while k < Q + 1

    PCspikes = zeros([1 12000]);

    for i = 1:12000
        Hz = wave(i);
        p = Hz/20000;
        r = rand;
        if r <= p
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 12000]);
    
    i = 1;
    while i < 12000
        if i < length(PCspikes)-(length(g5_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g5_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g5_influxUnit)-1)+g5_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCpopulation(k,:) = PCcurrent;

    k = k+1;

end

h = 1;

allremasters = zeros([1 12000]);

while h < N + 1

    j = 1;
    master = zeros([1 12000]);
    
    while j < S + 1
    
        master = master + PCpopulation(randi(Q),:); %
        j = j + 1;
    
    end
    
    remastered = integrate_all_intervals(master);
    
%     subplot(1,1,1)
%     x = 1:12000;
    if h < 11
        plot(x,remastered,'Color',([.6 .6 .6]))%,'LineWidth',1.4)
    end
%     xlabel('ms'); ylabel('current'); title('40:1 convergence, 50-200 Hz, 1 ms rolling av');
%     xlim([3600 11600]); xticks([3600 4100 4600 ...
%         5100 5600 6100 6600 7100 7600 8100 8600 9100 9600 10100 10600 11100 ...
%         11600]); 
%     xticklabels([0 25 50 75 100 125 150 175 200 225 250 275 300 325 350 375 ...
%         400])
%     ylim([0 30]); yticks([0 10 20 30]); yticklabels([0 10 20 30])
%     set(gca,'FontSize',10)
    
    hold on
    
    h = h + 1;

    allremasters = [allremasters; remastered];

end

allremastersmean = mean(allremasters);
plot(x,allremastersmean,'Color',([.6 .6 .6]),'LineWidth',1.5)
